using System.ComponentModel;

using Android.Gms.Maps;
using Android.Gms.Maps.Model;
using Android.OS;
using Android.Views;
using MapModalRepro.Controls;
using Java.Lang;
using Java.Util;
using Xamarin.Forms;
using Xamarin.Forms.Platform.Android;
using MapModalRepro.Droid.Controls;

[assembly: ExportRenderer(typeof(MyMap), typeof(MyMapRenderer))]

namespace MapModalRepro.Droid.Controls
{
    public class MyMapRenderer : ViewRenderer, GoogleMap.IOnCameraChangeListener
    {
        public MyMapRenderer()
        {
            AutoPackage = false;
        }

        static Bundle s_bundle;
        private bool _disposed;
        private bool _touchOver;

        internal static Bundle Bundle
        {
            set { s_bundle = value; }
        }

        protected GoogleMap NativeMap => ((MapView)Control).Map;

        protected MyMap Map => (MyMap)Element;

        private LatLng _currentPosition;
        private float _zoomLevel = 15f;

        public override SizeRequest GetDesiredSize(int widthConstraint, int heightConstraint)
        {
            return new SizeRequest(new Size(Context.ToPixels(40), Context.ToPixels(40)));
        }



        protected override void OnElementChanged(ElementChangedEventArgs<Xamarin.Forms.View> e)
        {
            base.OnElementChanged(e);
            SetOnTouchListener(this);

            var mapView = new MapView(Context);
            mapView.OnCreate(s_bundle);
            mapView.OnResume();

            SetNativeControl(mapView);

            if (e.OldElement != null)
            {
                MessagingCenter.Unsubscribe<MyMap, MyMapSpan>(this, MyMap.MoveMapMessageName);
            }

            var map = NativeMap;
            if (map != null)
            {
                map.SetOnCameraChangeListener(this);
                map.UiSettings.SetAllGesturesEnabled(false);
                map.UiSettings.ZoomControlsEnabled = false; // Zoom controls are annoying and IOS does it by default
                map.UiSettings.ZoomGesturesEnabled = Map.HasZoomEnabled;
                map.UiSettings.ScrollGesturesEnabled = Map.HasScrollEnabled;
                map.MyLocationEnabled = map.UiSettings.MyLocationButtonEnabled = Map.IsShowingUser;
            }


            MessagingCenter.Subscribe<MyMap, MyMapSpan>(this, MyMap.MoveMapMessageName, OnMoveMapMessage, Map);

        }

        void OnMoveMapMessage(MyMap s, MyMapSpan a)
        {
            MoveToRegion(a, a.AnimateTransitions);
        }

        protected override void OnElementPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            base.OnElementPropertyChanged(sender, e);


            var gmap = NativeMap;
            if (gmap == null)
                return;

            if (e.PropertyName == MyMap.IsShowingUserProperty.PropertyName)
                gmap.MyLocationEnabled = gmap.UiSettings.MyLocationButtonEnabled = Map.IsShowingUser;
            else if (e.PropertyName == MyMap.HasScrollEnabledProperty.PropertyName)
                gmap.UiSettings.ScrollGesturesEnabled = Map.HasScrollEnabled;
            else if (e.PropertyName == MyMap.HasZoomEnabledProperty.PropertyName)
            {
                gmap.UiSettings.ZoomGesturesEnabled = Map.HasZoomEnabled;
            }
        }

        void MoveToRegion(MyMapSpan span, bool animate)
        {
            var map = NativeMap;
            if (map == null)
                return;

            span = span.ClampLatitude(85, -85);
            CameraUpdate update =
                CameraUpdateFactory.NewLatLngZoom(new LatLng(span.Center.Latitude, span.Center.Longitude), _zoomLevel);

            try
            {
                if (animate)
                    map.AnimateCamera(update);
                else
                    map.MoveCamera(update);
            }

            catch (IllegalStateException exc)
            {
                System.Diagnostics.Debug.WriteLine("MoveToRegion exception: " + exc);
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && !_disposed)
            {
                _disposed = true;

                var mapModel = Element as MyMap;
                if (mapModel != null)
                {
                    MessagingCenter.Unsubscribe<MyMap, MyMapSpan>(this, MyMap.MoveMapMessageName);
                }

                var gmap = NativeMap;
                if (gmap == null)
                    return;
                gmap.MyLocationEnabled = false;
                gmap.Dispose();
            }

            base.Dispose(disposing);
        }

        public void OnCameraChange(CameraPosition position)
        {
            if (_currentPosition == null)
            {
                _currentPosition = position.Target;
            }
            // new latLong
            LatLng newPosition = position.Target;
            // new zoom
            if (position.Zoom > 1)
            {
                _zoomLevel = position.Zoom;

            }
            if (_touchOver && (newPosition != _currentPosition))
            {
                UpdatePosition(newPosition);
                _touchOver = false;
            }


        }

        private void UpdatePosition(LatLng pos)
        {
            Position newPosition = new Position(pos.Latitude, pos.Longitude);
            if (_touchOver)
            {
                ((MyMap)Element).MapCentreChangeCommand?.Execute(newPosition);
            }
        }

        public override bool OnTouchEvent(MotionEvent e)
        {
            var map = NativeMap;
            if (map != null)
            {
                if (MotionEventActions.Up == e.Action || MotionEventActions.Cancel == e.Action)
                {
                    _touchOver = true;
                }
                else
                {
                    _touchOver = false;
                }
                return true;
            }
            return true;
        }





        public override bool OnInterceptTouchEvent(MotionEvent ev)
        {
            OnTouchEvent(ev);
            return base.OnInterceptTouchEvent(ev);
        }
    }

}