﻿using System;
using System.Windows.Input;
using Xamarin.Forms;

namespace MapModalRepro.Controls
{
    public class MyMap : Xamarin.Forms.View
    {

        public static readonly BindableProperty HasZoomEnabledProperty = BindableProperty.Create("HasZoomEnabled", typeof(bool), typeof(MyMap), true);
        public static readonly BindableProperty HasScrollEnabledProperty = BindableProperty.Create("HasScrollEnabled", typeof(bool), typeof(MyMap), true);
        public static readonly BindableProperty MapTypeProperty = BindableProperty.Create("MapType", typeof(MapType), typeof(MyMap), default(MapType));
        public static readonly BindableProperty IsShowingUserProperty = BindableProperty.Create("IsShowingUser", typeof(bool), typeof(MyMap), default(bool));
        public static readonly BindableProperty MapCentreChangeCommandProperty = BindableProperty.Create(nameof(MapCentreChangeCommand), typeof(ICommand), typeof(MyMap));


        public MyMapSpan LastMoveToRegion { get; set; }

        public const string MoveMapMessageName = "MoveMapToRegion";


        public ICommand MapCentreChangeCommand
        {
            get
            {
                return GetValue(MapCentreChangeCommandProperty) as ICommand;
            }
            set
            {
                SetValue(MapCentreChangeCommandProperty, value);
            }
        }


        public bool HasZoomEnabled
        {
            get { return (bool)GetValue(HasZoomEnabledProperty); }
            set { SetValue(HasZoomEnabledProperty, value); }
        }

        public bool HasScrollEnabled
        {
            get { return (bool)GetValue(HasScrollEnabledProperty); }
            set { SetValue(HasScrollEnabledProperty, value); }
        }

        public MapType MapType
        {
            get { return (MapType)GetValue(MapTypeProperty); }
            set { SetValue(MapTypeProperty, value); }
        }


        public bool IsShowingUser
        {
            get { return (bool)GetValue(IsShowingUserProperty); }
            set { SetValue(IsShowingUserProperty, value); }
        }

        public void MoveToRegion(MyMapSpan mapSpan, bool animate)
        {
            if (mapSpan == null)
                throw new ArgumentNullException(nameof(mapSpan));
            LastMoveToRegion = mapSpan;
            mapSpan.AnimateTransitions = animate;
            MessagingCenter.Send(this, MoveMapMessageName, mapSpan);
            MapCentreChangeCommand?.Execute(mapSpan.Center);
        }

    }

    public enum MapType
    {
        Street,
        Satellite,
        Hybrid
    }
}
