﻿using System;

namespace MapModalRepro.Controls
{
    public class MyMapSpan
    {
        const double EarthRadiusKm = 6371;
        const double EarthCircumferenceKm = EarthRadiusKm * 2 * Math.PI;
        const double MinimumRangeDegrees = 0.001 / EarthCircumferenceKm * 360; // 1 meter
        public bool AnimateTransitions { get; set; }

        public MyMapSpan(Position center, double latitudeDegrees, double longitudeDegrees)
        {
            Center = center;
            LatitudeDegrees = Math.Min(Math.Max(latitudeDegrees, MinimumRangeDegrees), 90.0);
            LongitudeDegrees = Math.Min(Math.Max(longitudeDegrees, MinimumRangeDegrees), 180.0);
            AnimateTransitions = true;
        }


        public static MyMapSpan FromCenterAndRadius(Position center, double radius)
        {
            return new MyMapSpan(center, 2 * DistanceToLatitudeDegrees(radius), 2 * DistanceToLongitudeDegrees(center, radius));
        }

        static double DistanceToLatitudeDegrees(double distance)
        {
            return distance / EarthCircumferenceKm * 360;
        }

        static double DistanceToLongitudeDegrees(Position position, double distance)
        {
            double latCircumference = LatitudeCircumferenceKm(position);
            return distance / latCircumference * 360;
        }

        public Position Center { get; }

        public double LatitudeDegrees { get; }

        public double LongitudeDegrees { get; }





        public MyMapSpan ClampLatitude(double north, double south)
        {
            north = Math.Min(Math.Max(north, 0), 90);
            south = Math.Max(Math.Min(south, 0), -90);
            double lat = Math.Max(Math.Min(Center.Latitude, north), south);
            double maxDLat = Math.Min(north - lat, -south + lat) * 2;
            Position pos = new Position(lat, Center.Longitude);
            return new MyMapSpan(pos, Math.Min(LatitudeDegrees, maxDLat), LongitudeDegrees);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            return obj is MyMapSpan && Equals((MyMapSpan)obj);
        }


        public override int GetHashCode()
        {
            unchecked
            {
                int hashCode = Center.GetHashCode();
                hashCode = (hashCode * 397) ^ LongitudeDegrees.GetHashCode();
                hashCode = (hashCode * 397) ^ LatitudeDegrees.GetHashCode();
                return hashCode;
            }
        }

        public static bool operator ==(MyMapSpan left, MyMapSpan right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(MyMapSpan left, MyMapSpan right)
        {
            return !Equals(left, right);
        }

        public MyMapSpan WithZoom(double zoomFactor)
        {
            double maxDLat = Math.Min(90 - Center.Latitude, 90 + Center.Latitude) * 2;
            return new MyMapSpan(Center, Math.Min(LatitudeDegrees / zoomFactor, maxDLat), LongitudeDegrees / zoomFactor);
        }


        bool Equals(MyMapSpan other)
        {
            return Center.Equals(other.Center) && LongitudeDegrees.Equals(other.LongitudeDegrees) && LatitudeDegrees.Equals(other.LatitudeDegrees);
        }

        static double LatitudeCircumferenceKm(Position position)
        {
            return EarthCircumferenceKm * Math.Cos(position.Latitude * Math.PI / 180.0);
        }

        static double LatitudeDegreesToKm(double latitudeDegrees)
        {
            return EarthCircumferenceKm * latitudeDegrees / 360;
        }

        static double LongitudeDegreesToKm(Position position, double longitudeDegrees)
        {
            double latCircumference = LatitudeCircumferenceKm(position);
            return latCircumference * longitudeDegrees / 360;
        }
    }
}

